# -*- coding: UTF-8 -*-
# !/usr/bin/python
# @time     :2019/4/9 23:05
# @author   :Mo
# @function :SimBERT再训练BERT-base(NSP任务), UNILM的生成能力
# @reference:https://github.com/bojone/bert4keras
# 目前仅保证支持 Tensorflow 1.x + Keras <= 2.3.1 + bert4keras>=0.10.6。
# 具体用法请看 https://github.com/bojone/bert4keras/blob/8ffb46a16a79f87aa8cdf045df7994036b4be47d/bert4keras/snippets.py#L580


import os
os.environ["CUDA_VISIBLE_DEVICES"] = "0"
from bert4keras.snippets import sequence_padding, AutoRegressiveDecoder
from bert4keras.models import build_transformer_model
from bert4keras.tokenizers import Tokenizer
from bert4keras.backend import keras, K
import numpy as np
import time


# bert配置
# BERT_DIR = "D:/soft_install/dataset/bert-model/chinese_L-12_H-768_A-12"
# BERT_DIR = "D:/soft_install/dataset/bert-model/zuiyi/chinese_roberta_L-4_H-312_A-12_K-104"
# BERT_DIR = "D:/soft_install/dataset/bert-model/zuiyi/chinese_roberta_L-6_H-384_A-12_K-128"
# BERT_DIR = "D:/soft_install/dataset/bert-model/zuiyi/chinese_simbert_L-4_H-312_A-12"
# BERT_DIR = "D:/soft_install/dataset/bert-model/zuiyi/chinese_simbert_L-12_H-768_A-12"
BERT_DIR = "D:/soft_install/dataset/bert-model/zuiyi/chinese_simbert_L-6_H-384_A-12"

config_path = BERT_DIR + "/bert_config.json"
checkpoint_path = BERT_DIR + "/bert_model.ckpt"
dict_path = BERT_DIR + "/vocab.txt"
maxlen = 128


# 建立分词器
tokenizer = Tokenizer(dict_path, do_lower_case=True)  # 建立分词器

# 建立加载模型
bert = build_transformer_model(
    config_path,
    checkpoint_path,
    with_pool='linear',
    application='unilm',
    return_keras_model=False,
)

encoder = keras.models.Model(bert.model.inputs, bert.model.outputs[0])
seq2seq = keras.models.Model(bert.model.inputs, bert.model.outputs[1])


# class SynonymsGenerator(AutoRegressiveDecoder):
#     """seq2seq解码器
#     """
#     @AutoRegressiveDecoder.set_rtype('probas')
#     def predict(self, inputs, output_ids, states):
#         token_ids, segment_ids = inputs
#         token_ids = np.concatenate([token_ids, output_ids], 1)
#         segment_ids = np.concatenate([segment_ids, np.ones_like(output_ids)], 1)
#         # return self.last_token(seq2seq).predict([token_ids, segment_ids])
#         return seq2seq.predict([token_ids, segment_ids])[:, -1]
#
#     def generate(self, text, n=1, topp=0.95):
#         token_ids, segment_ids = tokenizer.encode(text, max_length=maxlen)
#         output_ids = self.random_sample([token_ids, segment_ids], n, topp=topp)  # 基于随机采样
#         return [tokenizer.decode(ids) for ids in output_ids]


class SynonymsGenerator(AutoRegressiveDecoder):
    """seq2seq解码器
    """
    @AutoRegressiveDecoder.wraps(default_rtype='probas')
    def predict(self, inputs, output_ids, step):
        token_ids, segment_ids = inputs
        token_ids = np.concatenate([token_ids, output_ids], 1)
        segment_ids = np.concatenate([segment_ids, np.ones_like(output_ids)], 1)
        return self.last_token(seq2seq).predict([token_ids, segment_ids])

    def generate(self, text, n=1, topp=0.95, mask_idxs=[]):
        token_ids, segment_ids = tokenizer.encode(text, maxlen=maxlen)
        for i in mask_idxs:
            token_ids[i] = tokenizer._token_mask_id
        output_ids = self.random_sample([token_ids, segment_ids], n, topp=topp)  # 基于随机采样
        return [tokenizer.decode(ids) for ids in output_ids]


synonyms_generator = SynonymsGenerator(start_id=None, end_id=tokenizer._token_end_id, maxlen=maxlen)


def gen_synonyms(text, n=100, k=20):
    """"含义： 产生sent的n个相似句，然后返回最相似的k个。
    做法：用seq2seq生成，并用encoder算相似度并排序。
    """
    r = synonyms_generator.generate(text, n)
    r = [i for i in set(r) if i != text]
    r = [text] + r
    X, S = [], []
    for t in r:
        x, s = tokenizer.encode(t)
        X.append(x)
        S.append(s)
    X = sequence_padding(X)
    S = sequence_padding(S)
    Z = encoder.predict([X, S])
    Z /= (Z**2).sum(axis=1, keepdims=True)**0.5
    argsort = np.dot(Z[1:], -Z[0]).argsort()
    return [r[i + 1] for i in argsort[:k]]


if __name__ == '__main__':
    datas = [{"text": "平乐县，古称昭州，隶属于广西壮族自治区桂林市，位于广西东北部，桂林市东南部，东临钟山县，南接昭平，西北毗邻阳朔，北连恭城，总面积1919.34平方公里。"},
             {"text": "平乐县主要旅游景点有榕津千年古榕、冷水石景苑、仙家温泉、桂江风景区、漓江风景区等，平乐县为漓江分界点，平乐以北称漓江，以南称桂江，是著名的大桂林旅游区之一。"},
             {"text": "印岭玲珑，昭水晶莹，环绕我平中。青年的乐园，多士受陶熔。生活自觉自治，学习自发自动。五育并重，手脑并用。迎接新潮流，建设新平中"},
             {"text": "桂林山水甲天下, 阳朔山水甲桂林"},
             {"text": "三国一统天下"},
             {"text": "世间万物皆系于其上"},
             {"text": "2020年真是一个糟糕的年代, 进入20年代，新冠爆发、经济下行，什么的都来了。"},
             {"text": "仿佛一切都变得不那么重要了。"},
             {"text": "苹果多少钱一斤"}
             ]
    time_start = time.time()
    for da in datas:
        text = da.get("text", "")
        res = gen_synonyms(text)
        print(res)
    time_total = time.time() - time_start
    print("time_total:{}".format(time_total))
    print("time_per:{}".format(time_total / len(datas)))
    print("time_one:{}".format(time_total / len(datas) / 20))

    while True:
        print("请输入:")
        text = input()
        res = gen_synonyms(text)
        print(res)


"""
['广西昭阳县，古称昭州，古称昭州，东连昭阳，西连，东连钟乐县，西西连地围到广西高度，望到桂市市，是昭城县段，北开连关，西连南端，南望南中关是永北到广', 
'元判随过昭山，东部，西连昭和，东连钟山，西。南离桂阳，东连钟镇、东风宁，贵州，西国六别公是西西，西关陇南和柔州是正经南南与元城，',
'桂林区与西连曹甸区，隶怎样关系邹庄区，东和县，东湖阳公合意，桂公襄公之州，西连南部，西虞公绍县，南南广字的地方，建裕州，南阳，北宫、南立中北部，平平到八公里南南南之耳旺侯的末阳州，东洼街，西省是横主', '广西四川区，西贯泗水，南称昭州，东连钟山，东连昭阳县，东连恭庄，东川山山国运是北北，使秦山，东坐坐南阳水水州，东归大都地人汉昌阳安南国，天州, 北洋与孝州, 晋都京东哪在在了', '长途为中北中广西壮族，南到昭阳，远游长卢县西北麓，西连延沙，南昌，市东关和北北公立，西营盘州，南江南州三', '南公西半丘襄节宜，以下东围于广西州、西带雁安、西陆连合三部，南支于哪都是接哪个县，建义城县，中是桂林，时台至公司南南闹直接看桂城，宾旗首关的齐称是公侯?', '昭合县，鹿头真迁庄, 西接昭阳及西连衡、桂襄三北区, 西管与昭公至广西平乐县, 西连连州、西西面赤柱，西千数依无农镇畅都是介绍广水五亭和宽春, 南兰各称九株都是公公, 解奈昆虫州, 西断切, 西连镇归，位称昌邑, 建丘城区到, 西西州、溪闸，西城景观阳视哪几公村为高门, 、州', '西接昭阳，平乐县，从桂林，桂林，南连衡阳阳，南连忠城，南接昭阳，东连桂阳，西西连水下可予攻途的规定，请点州北江, 东江，庄北，扶溪山', '桂林南带西途列以及中间中区, 北直到平乐县, 西邻三洛溪，有标值，韶关陵侯吗? 西北县（南五分别是四分别称石属关）的东横拨纬部?', '从桂林到昭州，西观那个城市，东接中湖，西连钟市，东接钟峰，南坐诸州到丰城，顺便建约楼多少架接相相式，东纬南上立，景和桂州，北路，时光线，桂阳，从武庙', '扬州市，西连钟与昭汇，西藏住住君丰南洋人坐属于桂林到武都和北京市东环座吗？是单位就江家五批售基五音堰县，西连嘉州在桂林是在桂山分别在佛于于是北北公里', '长守部，西连昭阴阳朔，西接昭央公县，西到长江市，西连钟山是直线吗？？南？长安，南山塔古都，周魏，山峰州，西省是周正州，谁到长宁分别代说农村县，古藏风景与昭永吗？？', '中南南三口，东北偏北部，东关到桂林，东连侯府，长马一平朔，北接昭肃，东黄金关旗，西藏，南江、阳相鹤湾、电兴，南连晋宁市, 北阳长州、连吉河', '励途能承主至壮族，昭襄, 从北连到昭丽县是在昆明县, 直接昭观凌县, 东连顺与林浦一交运吗，一般, 连别称、海州, 湘，关苏, 山州镇重江吗? 南京还是陇北, 南州？', '桂林市，南接昭平，平乐县, 南接昭阳，南站？西接邹岛，西连忠', '昭林，平乐县，西直展政府，北连宣放，东山，昭州、西连川及西西看评市后边那个区域内是南南仙陵县，平乐县在南南心定曰楼及兴放就扬，西开北斜视都开拘，桂公为什么国西一在贵州，吉吉吉，神通、昭凉(北除了正州，南南地有公立、辽岭公州, 东成陵厉害, 以河用国商，三道弄阳', '景作园西的历史照片值是广西东北部，平乐县，南部，东接钟山，东南的，围封桂林，西段是否和广西景思，北张则中的郫乡山，西南，南部，陕西，东北齐字俗？', '襄阳平乐县，古称昭州，北连主要区，从南接其中西和南都从天河县，南结衡港市, 西连楚侯区，西美公主要管管境是昭和丞，从从开公转大二乡到恭陵，路西, 榆归在南邑, 西公、郸阳到关河北充台的总北郸, 是从中的, 都是离广州有丞镇里州到侯侯, 咸，山阳, ，，北阳市, 西西至北、', '西连忻集市, 东接昭阳城, 西接昭端县, 西连扬桥, 北连雁城, 朝朝, 西连守望, 西丘县, 南连陆州, 南南镇, 常南至乐、北滨的乐在矩定区, 北部旺邑、皇陵和明河县, 温阳县、县邮属县, 西平, 北门、南侯邯郸, 北娄水镇、和中常天地, 北州、山东、南同时时, 侯东进都进贡县, 部。属', '乐县，隶服守应靖县，南接广西长沙，西连衡州，西省，东曲经里，西连雁和，北麓都先唐县，菏连，南南浦东县，东连唐和桂成段，时阀，部入阳阳, 南连邹岳贞山县，北连阳朔，安日, 北河，西至钟节公的玄合，西阳节节、汾韶山是公州, 和昭州，大关利桂阴，南南？哪边，曲守一天，']
['九南仙家古景区、鹭鸥主要景点、荔枝寺、仙乡与贵龙之景景点、玉山、桂县、秋风景的主要旅游景点、热誉观点、长瀑诗貌、客客城、天江水、瀑江、韶鸣、吴杨、鹤种，三兆、三角公', '榕津千年古景区，仙家岩道，常开酒，望迎桂江分界在关网上享悦城，天桑、榕景区，夏天洒江水村，古年古古诗要好景景，温度，寒秋，大揭银花，，谁的老境属哪在平乐县五大庙。会工陈述，宁宁，皇首开始仙仙, 》，开宁滨有，月庆的？好为锁花山，梅年和仙玉栈（市巷，谢谢欣店的', '苍海镇、榕津千年古映景区，望有景点，从漓江、枫景、沧南，现在历史、天津、沧古、纺石到南、九台、十头津、六十景景、仙家、夏海夏光，沧州的温泉、天海景或中', '纵横江与纵横济源古和风景区有咋样的旅游景点分东西，雁乐县的，长途旅游，长安铜县、仙家段分山关隔口到长乐有浩径水海海游，逍遥地方，长宁温泉、风景嘉宾？', '长途旅游，清扬碧峰花都是十大舜海海，三者分界，八一湖里长陵千年或凡天水仙和遇有桂江、云云坛、榕津的首江属山，谢圣的中代，九枫峡龙峨吒、颜满。一境区适合养羊，仙莲地铁街道下古故事、桂堰，仙和石是中？三个翠江，几水，苍兮远景，寒缘景景，诸元贵峰，明龙的你分别', '珍过地铁水流和竹耳、风光景区、长州至雪泉分区、仙家温泉、木音竹石、仙宁、九坪、平寿亭、长城、风沙、景景至去、吴路古场、流水茶、电山烟笛、江水、无水流寺有皇珠与玉风景是怎样的吗? 贵台、九桥、御河、钟频、的! 哪里有白山和竹、海门、温酒电头，花叶、垂花贵镇的景景', '作四个香港天峰海、仙家温泉、温泉、仙家分别是，榕津、茅亭中间纵路与桑梓、华海江、五字间有海海千万家、古景、君江、到乐、回渡、古、江，一定区，仙花、龙、贵江、久日坪、贵嘉古路而足的的哪些景点，是仙、汕、澜四和、长的故乡、鹤、贵、天柳的水，都是去外几四地目，揭', '黄桥至鹭江风景区，桂江，海滩仙家江南，仙家三院，有古代古风彩色龙眼，长江西鹤湖、桃桥、、、洪潭，贵熙，夏风关西，各距山，仙景，游，海缘冬天，仙江、长长春、江江、坪珠，，是不是、雨灼，凤，盛余白景看看', '季节景点绍宾区、松波云火木林、仙家温泉、仙家大雁到仙江湘湖、仙溪物游景点、开阳县、渝香路、陇铃、桐江、风风泉县有桂桂岛，桐光千江，扬沙日客景点、三眉江、流风景，仙棠海、冬雅、鹤里，雪夜感上，花叶墙、钟家竹、香江、耀侯、帆度，城东、谢晖旅游、峰夏、桥岸、杭区', '庆溪下游园和仙家杭州有长沙哪个旅游景点茂胜十二点，仙江海岸，暖城段和天津旅游景点，仙亭亭段有仙水有仙水乡里江西串线，大家观景景点，仙溪竹名叫昆游景区（镇火讯好说话岸季节有三点招出）的，这六盏山的，仙花天景，与川雨景景处在长夜景，晋报】沿岸水屿，白珠，录辉、', '平乐县、桂江与松暑，周边亦称至寒地，贵点，秋冬景点，松沙夜速、竹节、廊东、市期秋钟、松天江、鹤沙、海贺，周天仙桥，燕播长度波，哪个景区、窍侯、骏河，天天如是鑫仙、泉澳三云乡? 有四点分点是买柳曲路、鱼炬之来，地是称颜、观观景，瑜天源，段，总市，，花花天，', '贵阳仙家温泉到羊林结园，守山花君们，从寒草岛，仙家连云港，峨眉西游古梅木的旅游，白菜街道，隔誉万江道，庐岭天滩和广门与热园，乐寺到去无心天夏，多少，一里离了会在杭州，什么从东鹏，仙绵，石世节，河南风湖区的去，花花、陀河的旅游旅游景色，，仙，钟草、榕，花花花', '无天中弘湖仙家秋天、仙家孤栖西安、仙家高峰举三家瀑洒、仙冬风要在西湾花园中有景点、花光秋园、山州、曲传乐湘古云壶、逍遥方间、仙庭、贵贵算五二江、一首去、诸地属成这', '起源，中江仙家仙又是仙窑、羊坡秋季，同趣山, 开始，长长, 几侯地，为什么有些地方有棠口到桂公, 一港, 天府、红在九余州, 有宁乡、烂、云邦、潍坊、宁乡、鱼江、镇营的七州?', '桂江酒店与三旗九岛客观对比，长沙市晴山，仙家石景苑、寒阳各各有一点点一点，仙河仙古园、温泉、石壁、铜堰至至起墨苑，，西莞、重江，仙流，御泉、天叶、亮市、洪城，仙家娥乐、遇稽各是多少，雍熙，仙家和三蓝，楚山、井风是仙，海洋，流银水、和、寺、贵莲。谁的铁火、湘', '桂江、双江、榕津、华浪、青属，仙家中心仙江、鹤和、仙江、水、景观区，温水，宋虞，热北台、竹乡、大丽、凤峰、厦江、等、小寺的长享、热、鄱阳、、海水，铁桥、川海、海宝名的风景区，乐县、紫阳、铜桐、各，镇耳、峰、宝丽、珠有相邻，宋石、开关，多头，注山、大、川到峰', '黔西牧木酒城、桂江法乡、天下石景区、起香坡、凤庄、、温泉、温梦建合二5户区、地省舟松、云风门、鸿江、高北八联林、竹门和仙家古古城、贵庭、霞峰、三石、沧州、碧尔贵大属羊、东陵、摩棠、峨眉一地管国旗、景声、热清，荆门式季能通过川壶和仙，接媚阳，芳庆、帆、酒及', '大陆的紫阳市，至白云山花园，紫海昭珠市，风门成都，仙家古山桑家十大的、季洋江是桂城、丘陵是金翡虹、玉翠山，可爱，仙韶丽州，省泉小妙花，楚庆粤超, 怡鸿，雪河秋水，云南省景区，榕津无定，桐池顺江; 夏宁，铜吴春，紫江钟、花竹或同春春林的，花花、山花、佛余江，', '在在啥样的景点，平乐县旅游, 温泉、仙家、五金街、红花、季色、云峡、延宁、遮水、莲塘、长江, 三山市又古、贵水、桑河, 玉二江、花江镇下下的，铺山、茯洞、蓉江、水、是什么、燕角、大、百花、山场旅游景点、长美、印无、桂石、剃镜、和、望、什么、风、天水的枇篱点, 鸟、', '东方学院、三国镇、白水学院、仙家境境，榕江至西安、狐白长时铜古古景区和九月藏、清古、江千河、长规寺、天夏六村、仙仙校苑、仙门古古古古古景、竹巷哪有长兴音乐园、热门古说学学校、竹径、桂江分综心、暖光唱湖、大空节竹，逍江仙、山景丽长天，青立校五寒、南浔、车道、']
['人最优秀、山坐节，青年环绕我平中清关于培养活着待接领，建设新党中的看，平相赞的? 精彩一大善上。君', 
'黄冈女孩英国，青年的乐园，曾读书高考：周元创月温，书如彩的一样、自读一生中，岳山成成就的中，云同莲回爱，泰国源。作人读就创读，论花滴一注算故旷以花的，人工代理。青阳美溪禅了温温雅丽，泉种文回还，怎么么样、玲韵清静梦的，韶大的哪的还是中心交为者颜', 
'欣预制古筝，青年的乐园，复合将书票刻着；星话拱廊开始，生活自我自制彩韵举奖壶舞，待光', 
'作千年印岭玲珑，昭曲的中会，美观三部曲，十二星座创业, 时间几电能销嘘，多童才; 历届作文晚美刘约讲，青广韵', 
'印岭玲珑，昭水晶透析，青年，待奖一双五育并成是人用手动模仿继承心投资星宇光远赛为者，韶韶为热乐然的，国贸，电海游，池奉义的爱欣音，佛可观视育啊！敬崇点乐歌唱？乐是什么人的，什么景的。我，，都望评金金采与收文，家家家，钟吉磨，', 
'珍戒，观赏节，颂中感，佛教自发自由松舞；孩子会自觉自动文化感情诗们出色，生日共是多育兴的，如今水明沉松，清理郭尔景的星宇，多久，学一年生翻唱、说. 福福趣，嘉言发，活、远清的天话总弘语、理听，国，详细, 作一句歌方言，求用旅。多生中人亦尊福乐大暖的，是中气、不', '印岭玲珑紫澈水皎，照示我平，青年的乐园，葱~ 秋花多杨并起来. 拼音盘点、地美结构学·浙江大通彩有在说的哥表. . ，一九，，，、& # o o, 蜡饼', '青年我的青年青光青春青乐处相关，乐的秋夜木在长碧海. 平除除了中国美国，美而海洋~ 大省间抗战的明天，丹《水》、历史。；春是什么，我听月星星星池？', '林林红爷，青年的乐园中，我回忆，建设新潮流，孙婷怀划法的永远会强放，现在，谁能真的的祝福赛酒中的，谢谢张笑亮，留留了三星，还是祝带着了一个生成花并加重爱。诗音或者是中之不。谢谢了家家你过怀？？', '青年的恒和，珍贵的树功能，生活自觉优秀，开弄永久的神话歌曲，希望对面，多人词曲，五新轻不同', '摘补韶关青峰的欣赏，励志开发，大家对图片，五育并重读辩论，望百年组影，吉，意爆，描写论了钟光，过过近三个什么都干了，我帮帮看', '纪元圆得诗都上，节节刘雪家的四堂，留于阳光手术，铸背，手游，沈圆，画的青年，美中华洪长熙加历年池身，光光精装练年会，明烛一光，真阳献飞报。清底山，，是当奉播明明心，铜学院，教堂、的，真真了遇乐淡是敬刘高子歌的', '平登书，鹰山亭，紫珠市之声，照湖居校开大学，松索，祥秀梦境，美人光头，，花花在九美假釉抒感情此迷时想，阅我海藻情呵音画化，歌冠网感情艺韶茂（迷观观观观吗），感谢的给下赞成个人尊尊敬吗）', '文中，青年的乐园，恒盛伦，12层子旅景，新潮流，打算学习教师们，翻译四级对比，能很长的，生生过程，生育，终理行', '晚会，祝成就美貌，青年的乐园，多辰沉尽，沈阳清水讨坚，五育翔沈曹央', '爱青峰有多远，晓得，青峰的乐园，观大叙述青花的生活解压住，别人在）（你相源于', '深深深海, 青年的乐园，玉亭和五育并重, 生活自重的关系, 生活自己流和手间两个神情相聚吗？', '湛江红紫总合并述迎年，青年青年的乐园，耕摩串唱，生活自觉自筹。编度，入意不处旋。五教习待好有才乐天天利有一直是从一点的自缩望，水张心一琴久的》中中在名，，课读字，，，，谢谢我，吉好的？我怀口乐，我追美说的的。？？？？？？？？？？', '育园中的红铁花的人于绝面，开头如此。生活自觉怎么自由自来，思想如今，诗人最能力量，', '木遵的四大盘点，菊花，祥物，木央绵力，舞湖，武师，五育并我也，地陆都播。学习笔拼词和眼气，枫舞，花笔摇钟，光手地图有艺艺一词了，温力本气，树名唱。乐容讲讲，集候。效概，文歌钢说是真感受是胡迹罗心的什么，怎么歌曲，盐读，木藻立意了的。作情和小而足，木还是']

['桂林山水甲天下, 阳朔山水甲', '桂林山水甲天下，阳朔山水甲', '桂林山水甲天下, 阳朔山水甲天下', '桂林山水甲天下，阳朔山水甲桂林阳朔山水甲', '桂林桂林山水甲天下还是阳朔', '山水甲天下】阳朔山水甲: 桂林', '桂林山水甲天下, 阳朔山水甲月海水', '桂林山水甲天下', '桂林山水甲天下, 山水甲?', '桂林山水甲天下，阳朔山水甲热散', '桂林山水甲天下！', '桂林山水甲天下？', '桂林那山水甲甲天下', '桂林山水甲天下, 阳朔山水甲天下什么梗', '桂林山水甲天下、阳朔山水甲甲一斑', '桂林山水甲天下什么', '桂林山水甲天下，山水甲季', '桂林环形污水甲天下, 阳朔山水甲天下, 什么', '阳朔山水甲天下，桂林山水甲，桂林山水甲天下. 5情侣', '桂林山水甲天下，桂林天下桂林山水甲天下好怎样']
['三国一统，天下', '三国一统天下什么？', '三国一统天下是什么？', '三国一统天下的形势', '三国一统天下地吗', '三国一统天下大名', '三国一统天下的力量', '三国一统天下是什么时候', '三国一统天下的成语', '三国一统天下是什么意思', '三国一统天下是什么意思？', '三国一统天下各家意义', '一道三国一统天下大全四', '三国一统天下填什么', '三国一统天下是否文化', '三国一统天下三字', '三国一统天下是不是真的', '三国一统，统天下是做什么的？', '三国一统天下百分之五', '三国一统天下与三国一统相调吗？']
['世间万物皆系于其上!', '世间万物皆系于其中', '为什么世间万物皆系于其上？？', '世间万物皆系于其上详情', '百间万物皆系于其上', '世间万物皆系于其上怎么解释', '世间万物皆系于自上介绍', '世间万物皆系于其上什么意思?', '万物皆系于其上', '世间万物皆系于其上账', '万物皆系于其上！~', '世间万物皆系于这里', '求解：世间万物皆系于其上。', '世间万物皆系于其上。大四怎样.', '世间万物皆系于其下是什么意思', '每天, 世间万物皆系于其上哪里', '世间万物皆系于其上说明成语', '我们世间万物皆系于其上流', '世间万物皆系，于其上，将及从怎样？', '世间万物皆系于其上再亦是什么意思']
['2020年真是一个糟糕的年代, 进入包发，原因是什么', '2020年真是一个糟糕的年代, 进入20年代会怎么样?', '2017年国报真是糟糕的年代, 这个糟糕是什么意思呢', '已经是个糟糕的年代，已经清到20年代，是什么意思', '2016年怎么和2010年末是成为糟糕的年代，这是什么时候进的', '2016年思维表应该真是糟糕的年代；在20年代，什么时候呢？', '19年真的是最糟糕的年代，其几点对于荣誉和经济或者经济', '今天今天大家知道2016年是一个糟糕的年代？', '2017年到2019年今年是糟糕的年代吗？', '2017年真是一个糟糕的年代, 还是两个人进越进头有分别的', '2018年战略" 这年真是一个糟糕的年代; 怎样在2015年十五年代', '2020年真的是正反经？我为什么不进入呢？今后全是糟糕的年', '有人来过2015年真是是糟糕的时期吗', '人们，我中的青少年真是糟糕的年代》进入新冠爆', '今天开始了这回答，到哪里了，还是进入20年代新冠', '今年是什么年代这个年代，去战年现在什么样的年代进入了', '20年代，经济报告中知觉发为什么这么糟糕？', '，今年2019年发的是大年代，现在又那么糟糕的年代和大公主', '超过20年，也是不一样，新冠你力在爆发很糟糕吗', '2017年为什么新冠爆发了是每个的年代啊？']
['仿佛一切都变得比这么重要？', '仿佛一切都变得高又重要', '仿佛一切都变得重要了, 怎么调整过来', '你觉得仿佛一切都不很重要吗', '如果仿佛变得不那么重要。。', '仿佛变得不很重要了，还是都很重要？', '想变成仿佛一切这么重要！', '大家仿佛一切变得不那么重要了，美女背', '仿佛自己好像那么重要？一切就变的恐怖了么？', '仿佛真正变得很变那么重要', '仿佛一切都一切那么重要怎么解释', '仿佛真的变得那么重要了吗？', '仿佛一切是更重要了吗', '仿佛每次会变得不那么重要吗？', '仿佛一切这么重要吗', '仿佛都变得实，自从重要了吗？', '仿佛的变富，我也不那么重要了', '仿佛一切都变得太有趣了', '为什么仿佛一切不一样，只能变得变得高？', '仿佛学，真的变不这么重要了吗']
['苹果多少钱一斤？', '一斤苹果多少钱一斤', '苹果一斤多少钱', '一斤多少钱苹果', '苹果一斤多少钱？', '苹果是多少钱一斤？', '苹果多少钱一斤苹果价格', '苹果价格多少钱一斤', '苹果钱一斤多少钱', '买苹果用多少钱一斤', '苹果手机多少钱一斤', '苹果片多少钱一斤', '苹果卖家多少钱一斤', '我有一斤苹果多少钱一斤', '苹果多少钱一个', '苹果1万元的大概多少钱一斤', '苹果一盒买多少钱一斤', '苹果一般多少钱', '苹果7多少钱一斤', '苹果13套多少钱一斤']
win10-i7-cpu
time_total:1108.481211423874
time_per:123.1645790470971
time_one:6.158228952354856
"""

